@@ -3,7 +3,7 @@
 #include <stdexcept>
 
 QPDF_Destroyed::QPDF_Destroyed() :
-    QPDFValue(::ot_destroyed, "destroyed")
+    QPDFValue(::ot_destroyed)
 {
 }
 
@@ -28,9 +28,8 @@ QPDF_Destroyed::unparse()
     return "";
 }
 
-JSON
-QPDF_Destroyed::getJSON(int json_version)
+void
+QPDF_Destroyed::writeJSON(int json_version, JSON::Writer& p)
 {
     throw std::logic_error("attempted to get JSON from a QPDFObjectHandle from a destroyed QPDF");
-    return JSON::makeNull();
-}
+}
\ No newline at end of file
